<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2017 Amasty (https://www.amasty.com)
 * @package Amasty_Promo
 */


namespace Amasty\Promo\Model;

class Registry extends \Magento\Framework\Model\AbstractModel
{
    protected $_hasItems = false;
    protected $_locked = false;
    protected $_isHandled = [];

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $_productFactory;

    /**
     * Store manager
     *
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Amasty\Promo\Helper\Item
     */
    protected $promoItemHelper;

    /**
     * @var \Amasty\Promo\Helper\Messages
     */
    protected $promoMessagesHelper;

    public function __construct(
        \Magento\Checkout\Model\Session $resourceSession,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Amasty\Promo\Helper\Item $promoItemHelper,
        \Amasty\Promo\Helper\Messages $promoMessagesHelper
    ) {
        $this->_checkoutSession    = $resourceSession;
        $this->scopeConfig         = $scopeConfig;
        $this->_productFactory     = $productFactory;
        $this->_storeManager       = $storeManager;
        $this->promoItemHelper     = $promoItemHelper;
        $this->promoMessagesHelper = $promoMessagesHelper;
    }

    public function getApplyAttempt($ruleId)
    {
        if (isset($this->_isHandled[$ruleId])) {
            return false;
        }
        $this->_isHandled[$ruleId] = true;

        return true;
    }

    public function addPromoItem($sku, $qty, $ruleId)
    {
        if ($this->_locked) {
            return;
        }

        if (!$this->_hasItems) {
            $this->reset();
        }

        $this->_hasItems = true;
        $items = $this->getPromoItems();
        $autoAdd = false;

        $addAutomatically = $this->scopeConfig->isSetFlag(
            'ampromo/general/auto_add',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );

        if (is_array($sku) && count($sku) == 1) {
            $sku = $sku[0];
        }

        if (!is_array($sku)) {
            if ($addAutomatically) {

                $collection = $this->_productFactory->create()->getCollection()
                    ->addAttributeToSelect(['name', 'status', 'required_options'])
                    ->addAttributeToFilter('sku', $sku)
                    ->setPage(1, 1);

                $product = $collection->getFirstItem();

                $currentWebsiteId = $this->_storeManager->getWebsite()->getId();
                if (!is_array($product->getWebsiteIds())
                    || !in_array($currentWebsiteId, $product->getWebsiteIds())
                ) {
                    // Ignore products from other websites
                    return;
                }

                if (!$product || !$product->isInStock() || !$product->isSalable()) {
                    $this->promoMessagesHelper->addAvailabilityError($product);
                } else {
                    if (in_array($product->getTypeId(), ['simple', 'virtual'])
                        && !$product->getTypeInstance(true)->hasRequiredOptions($product)
                    ) {
                        $autoAdd = true;
                    }
                }
            }

            if (isset($items[$sku])) {
                $items[$sku]['qty'] += $qty;
            } else {
                $items[$sku] = [
                    'sku'      => $sku,
                    'qty'      => $qty,
                    'auto_add' => $autoAdd
                ];
            }
        } else {
            $items['_groups'][$ruleId] = [
                'sku' => $sku,
                'qty' => $qty
            ];
        }

        $this->_checkoutSession->setAmpromoItems($items);
    }

    public function getPromoItems()
    {
        $items = $this->_checkoutSession->getAmpromoItems();

        return $items ? $items : ['_groups' => []];
    }

    public function reset()
    {
        if ($this->_hasItems) {
            $this->_locked = true;
            return;
        }

        $this->_checkoutSession->setAmpromoItems(['_groups' => []]);
    }

    public function getLimits()
    {
        $allowed = null;
        $quote   = $this->_checkoutSession->getQuote();

        if ($quote->getId() > 0) {
            $allowed = $this->getPromoItems();
            foreach ($quote->getAllItems() as $item) {
                $sku = $item->getProduct()->getData('sku');

                if ($this->promoItemHelper->isPromoItem($item)) {
                    $ruleId = $this->promoItemHelper->getRuleId($item);

                    if (isset($allowed['_groups'][$ruleId])) {
                        if ($item->getParentItem()) {
                            continue;
                        }

                        $allowed['_groups'][$ruleId]['qty'] -= $item->getQty();
                        if ($allowed['_groups'][$ruleId]['qty'] <= 0) {
                            unset($allowed['_groups'][$ruleId]);
                        }
                    } else {
                        if (isset($allowed[$sku])) {
                            $allowed[$sku]['qty'] -= $item->getQty();

                            if ($allowed[$sku]['qty'] <= 0) {
                                unset($allowed[$sku]);
                            }
                        }
                    }
                }
            }
        }

        return $allowed;
    }

    public function deleteProduct($sku)
    {
        $deletedItems = $this->_checkoutSession->getAmpromoDeletedItems();

        if (!$deletedItems) {
            $deletedItems = [];
        }

        $deletedItems[$sku] = true;

        $this->_checkoutSession->setAmpromoDeletedItems($deletedItems);
    }

    public function restore($sku)
    {
        $deletedItems = $this->_checkoutSession->getAmpromoDeletedItems();

        if (!$deletedItems || !isset($deletedItems[$sku])) {
            return;
        }

        unset($deletedItems[$sku]);

        $this->_checkoutSession->setAmpromoDeletedItems($deletedItems);
    }

    public function getDeletedItems()
    {
        $deletedItems = $this->_checkoutSession->getAmpromoDeletedItems();

        if (!$deletedItems) {
            $deletedItems = [];
        }

        return $deletedItems;
    }
}
